from AccessControl import ClassSecurityInfo
from Globals import DTMLFile
from AccessControl.SecurityManagement import getSecurityManager
from App.class_init import default__class_init__ as InitializeClass
from Products.ZSQLMethods.SQL import SQL

from Products.PluggableAuthService.interfaces.plugins import IRoleAssignerPlugin
from Products.PluggableAuthService.interfaces.plugins import IRolesPlugin
from Products.PluggableAuthService.interfaces.plugins import IRoleEnumerationPlugin
from Products.PluggableAuthService.permissions import ManageUsers
from Products.PluggableAuthService.utils import classImplements

from Products.SQLPASPlugin.plugins.sql import SQLBase
from Products.SQLPASPlugin import config as sqlpasconfig

manage_addSQLRoleManagerForm = DTMLFile(
    "../zmi/RoleManagerForm", globals())

def manage_addSQLRoleManager(self, id, title='', sql_connection='', REQUEST=None):
    """Add a SQLRoleManager to a Pluggable Auth Service.
    """
        
    rm = SQLRoleManager(id, title, sql_connection)
    self._setObject(rm.getId(), rm)
    
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
                                '%s/manage_workspace'
                                '?manage_tabs_message='
                                'SQLRoleManager+added.'
                            % self.absolute_url())



class SQLRoleManager(SQLBase):
    """PAS plugin for managing roles in a SQL-based database. 
    """
    meta_type = 'SQL Role Manager'
    security = ClassSecurityInfo()

    _properties = (
        {'id':'roles_table', 'type': 'string', 'mode': 'w'},
        {'id':'roles_col_username', 'type': 'string', 'mode': 'w'},
        {'id':'roles_col_rolename', 'type': 'string', 'mode': 'w'},
    )

    def __init__(self, id, title=None, connection=None):

        self._id = self.id = id
        self.title = title
        self._connection = connection

        self.manage_changeProperties( \
            roles_table=sqlpasconfig.ROLES_TABLE,
            roles_col_username=sqlpasconfig.ROLES_COL_USERNAME,
            roles_col_rolename=sqlpasconfig.ROLES_COL_ROLENAME,
        )
    
    def addSQLQueries(self):
        """ Create the SQL methods objects based on the _properties values
        and the queries templates.
        """
        roles_table = self.getProperty('roles_table')
        roles_col_username = self.getProperty('roles_col_username')
        roles_col_rolename = self.getProperty('roles_col_rolename')
        schemaConfig = dict(roles_table=roles_table,
                            username_col=roles_col_username,
                            rolename_col=roles_col_rolename)
        
        #Copied from SQLPASPlugin.plugins.sql.SQLBase
        for id, title, params, sql in self.getSQLQueries():
            sql = sql % schemaConfig
            if not hasattr(self, id):
                o = SQL(id, title, self._connection, params, sql)
                self._setObject(id, o)        

    def getSQLQueries(self):
        """ Return a tuple of queries. Each query has id, title,
        parameters and the query template to be used by addSQLQueries.
        """
        return _SQL_QUERIES

    security.declareProtected( ManageUsers, 'assignRolesToPrincipal' )
    def assignRolesToPrincipal( self, roles, principal_id ):
        """ Assign a specific set of roles, and only those roles, to a principal.
        o no return value
        o inser and delete roles on the SQL Backend based on the roles parameter 
        """
        roles = [role_id for role_id in roles if role_id not in ('Authenticated',
                                                                 'Anonymous',
                                                                 'Owner')]

        actual_roles = self.getRolesForPrincipal(principal_id)
        #Remove actual roles that are not in the roles parameter
        for actual_role in actual_roles:
            if actual_role not in roles:
                self.sqlDelRoleForUser(username=principal_id,
                                          rolename=actual_role)
        #Insert new roles
        for role_id in roles:
            if role_id not in ('Authenticated','Anonymous','Owner'):
                self.doAssignRoleToPrincipal(principal_id, role_id)


    #
    #   IRoleAssignerPlugin implementation
    #
    def doAssignRoleToPrincipal(self, principal_id, role):
        """ Create a principal/role association in a Role Manager
        o Return a Boolean indicating whether the role was assigned or not
        """
        roles = self.getRolesForPrincipal(principal_id)
        if role in roles:
            return False 
        
        self.sqlAddRoleForUser(username=principal_id, rolename=role)
        return True

    #
    #   IRolesPlugin implementation
    #
    security.declarePrivate('getRolesForPrincipal')
    def getRolesForPrincipal(self, principal, request=None):
        """ principal -> ( role_1, ... role_N )
        o Return a sequence of role names which the principal has.
        o May assign roles based on values in the REQUEST object, if present.
        """
        #XXX: When the table and columns are not equal of the default values and
        #to left the admin user change these values.
        try:
            result = self.sqlRolesForUser(username=principal)
        except:
            return

        roles = []
        for role in result:
            roles.append(role[0])
        return tuple(roles)


classImplements(SQLRoleManager,
                IRoleAssignerPlugin,
                IRolesPlugin)

InitializeClass(SQLRoleManager)

_SQL_QUERIES = (
    (
     "sqlRolesForUser",
     "Return the list of roles for a given principal",
     "username",
     """
SELECT %(rolename_col)s as rolename
FROM %(roles_table)s
WHERE %(username_col)s=<dtml-sqlvar username type="string">
     """
    ),
    
    (
     "sqlAddRoleForUser",
     "Assign the role to the user",
     "username rolename",
     """
INSERT INTO %(roles_table)s (%(username_col)s, %(rolename_col)s)
VALUES (<dtml-sqlvar username type="string">,
        <dtml-sqlvar rolename type="string">)
     """
    ),
    (
     "sqlDelRoleForUser",
     "Remove que role for a given user",
     "username rolename",
     """
DELETE FROM %(roles_table)s
WHERE %(username_col)s=<dtml-sqlvar username type="string">
AND   %(rolename_col)s=<dtml-sqlvar rolename type="string">
     """
    ),

)

