from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from OFS.PropertyManager import PropertyManager
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo
from Products.ZSQLMethods.SQL import SQL

from Products.SQLPASPlugin.utils import log, logEx
from Products.SQLPASPlugin import config as sqlpasconfig

manage_options = list(BasePlugin.manage_options)
for opt in Folder.manage_options:
    if opt not in manage_options:
        manage_options.append(opt)       
manage_options = tuple(manage_options)

class SafeSQLQuery(Folder):
    class Void:
        def tuples(self):
            return ()
        def dictionaries(self):
            return ()
    
    def __init__(self, query):
        self._setObject('query', query)
    
    def __call__(self, *args, **kwargs):
        try:
            return self.query(*args, **kwargs)
        except Exception, e:
            logEx("Something went wrong while trying to use the query %r." %
                  self.query)
            return self.Void()

class SQLBase(Folder, BasePlugin):

    security = ClassSecurityInfo()


    _properties = (
        {'id':'users_table', 'type': 'string', 'mode': 'w'},
        {'id':'users_col_username', 'type': 'string', 'mode': 'w'},
        {'id':'users_col_password', 'type': 'string', 'mode': 'w'},

        {'id':'col_mapping', 'type': 'lines', 'mode': 'w'},
    )

    def __init__(self, id, title=None, connection=None):

        self._id = self.id = id
        self.title = title

        self._connection = connection
        
        # Set defaults
        self.manage_changeProperties(
            users_table=sqlpasconfig.USERS_TABLE,
            users_col_username=sqlpasconfig.USERS_COL_USERNAME,
            users_col_password=sqlpasconfig.USERS_COL_PASSWORD,
            col_mapping='',
        )

        self.addSQLQueries()

    def manage_changeProperties(self, **kwargs):
        """Ensure the SQL methods get regenerated
        """

        self.delSQLQueries()
        PropertyManager.manage_changeProperties(self, **kwargs)
        self.addSQLQueries()

    def manage_editProperties(self, REQUEST):
        """Ensure the SQL methods get regenerated
        """
        
        self.delSQLQueries()
        res = PropertyManager.manage_editProperties(self, REQUEST)
        self.addSQLQueries()
        return res

    def delSQLQueries(self):
        sqllist=self.objectIds('Z SQL Method')
        self.manage_delObjects(ids=sqllist)

    def addSQLQueries(self):
        users_table = self.getProperty('users_table')
        users_col_username = self.getProperty('users_col_username')
        users_col_password = self.getProperty('users_col_password')
        schemaConfig = {
                            'users_table': users_table,
                            'username_col': users_col_username,
                            'password_col': users_col_password,
                       }
        for id, title, params, sql in self.getSQLQueries():
            sql = sql % schemaConfig
            if not hasattr(self, id):
                o = SQL(id, title, self._connection, params, sql)
                self._setObject(id, SafeSQLQuery(o))
                
    def getSQLQueries(self):
        return ()

    
    #
    #   ZMI
    #
    all_meta_types = (
        {'name': 'Z SQL Method', 
         'action': 'manage_addProduct/ZSQLMethods/add' },
    )
    manage_options = manage_options


