import sha
import plugin_exceptions
import types

from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from App.class_init import default__class_init__ as InitializeClass

from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.interfaces.plugins \
    import IAuthenticationPlugin
from Products.PluggableAuthService.interfaces.plugins \
    import IUserEnumerationPlugin
from Products.PluggableAuthService.interfaces.plugins \
    import IUserAdderPlugin

from Products.PluggableAuthService.permissions import ManageUsers
from Products.PluggableAuthService.permissions import SetOwnPassword

from Products.SQLPASPlugin.utils import log, logEx
from Products.SQLPASPlugin import config as sqlpasconfig

# TODO: fix the dependency on PlonePAS
from Products.PlonePAS.interfaces.plugins import IUserManagement

from Products.SQLPASPlugin.plugins.sql import SQLBase

manage_addSQLUserManagerForm = DTMLFile(
    "../zmi/UserManagerForm", globals())

def manage_addSQLUserManager(self, id, title='', sql_connection='', REQUEST=None):
    """Add a SQLUserManager to a Pluggable Auth Service.
    """

    um = SQLUserManager(id, title, sql_connection)
    self._setObject(um.getId(), um)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
                                '%s/manage_workspace'
                                '?manage_tabs_message='
                                'SQLUserManager+added.'
                            % self.absolute_url())

class SQLUserManager(SQLBase):
    """ PAS plugin for managing users in a SQL-based database.
    """

    meta_type = 'SQL User Manager'

    security = ClassSecurityInfo()

    def __init__(self, id, title=None, connection=None):

        SQLBase.__init__(self, id, title, connection)

        self._id = self.id = id
        self.title = title

    
    def doChangeUser(self, login, password, **kw):
        self.updateUserPassword(login, login, password)
        
    def doDeleteUser(self, login):
        raise NotImplementedError("This plugin does not yet delete users")


    #
    #   IAuthenticationPlugin implementation
    #
    security.declarePrivate('authenticateCredentials')
    def authenticateCredentials(self, credentials):
        """ See IAuthenticationPlugin.

        o We expect the credentials to be those returned by
          ILoginPasswordExtractionPlugin.
        """
        login = credentials.get( 'login' )
        password = credentials.get( 'password' )
        
        if login is None or password is None:
            return None

        user = self.getUserInfo(login)

        if user and (user['password'] == password or \
            user['password'] == sha.sha(password).hexdigest()):
            
            return login, login

        return None

    #
    #   IUserEnumerationPlugin implementation
    #
    security.declarePrivate( 'enumerateUsers' )
    def enumerateUsers( self
                      , id=None
                      , login=None
                      , exact_match=False
                      , sort_by=None
                      , max_results=None
                      , **kw
                      ):

        """ See IUserEnumerationPlugin.
        """
        
        ids = id
        if type(ids) not in (types.TupleType, types.ListType):
            ids = (ids,)
        logins = login
        if type(logins) not in (types.TupleType, types.ListType):
            logins = (logins,)
        
        if len(ids) == 1 and ids[0] is not None and login is None:
            res = self.sqlLoadUser(username=ids[0])
        elif len(logins) == 1 and logins[0] is not None and id is None:
            res = self.sqlLoadUser(username=logins[0])
        else:       
            res = self.sqlLoadAllUsers()
            log('enumerateUsers(): calling all: logins=%s; ids=%s;' % (logins, ids))
        
        tuples = res.tuples()
        all = []
        x = 0
        done = False
        while not done and x < len(tuples):
            record = tuples[x]
            data = {
                'id': record[0],
                'login': record[0],
                'password': record[1],
                'pluginid': self.getId(),
            }
            
            if (id is None and login is None) \
                    or data['id'] in ids or data['login'] in logins:
                all.append(data)
            
            if max_results is not None and len(all) == max_results:
                done = True
            
            x += 1
        
        return tuple(all)
        

    #
    #   IUserAdderPlugin implementation
    #
    security.declarePrivate( 'doAddUser' )
    def doAddUser( self, login, password ):
        try:
            self.addUser(login, login, password)
        except KeyError:
            return False
        return True

    #
    #   IUserManager interface
    #
    security.declareProtected(ManageUsers, 'listUserIds')
    def listUserIds(self):
        res = self.sqlLoadAllUsers()
        tuples = res.tuples()

        all = [ x['username'] for x in tuples ]
        
        return tuple(all)

    security.declareProtected(ManageUsers, 'getUserInfo')
    def getUserInfo(self, user_id):
        """ 
        """
        
        try:
            res = self.sqlLoadUser(username=user_id)
        except:
            logEx("Error while trying to query user")
            
        tuples = res.tuples()
        if len(tuples) > 1:
            raise plugin_exceptions.UnexpectedResultsException \
                ("The username column is not unique, got back multiple " \
                 "records for user '%s'" % user_id)
        
        data = None
        
        if len(tuples) > 0:
            record = tuples[0]
            data = {
                'id': record[0],
                'login': record[0],
                'password': record[1],
                'pluginid': self.getId(),
            }

        return data

    security.declareProtected( ManageUsers, 'listUserInfo' )
    def listUserInfo(self):
        res = self.sqlLoadAllUsers()
        tuples = res.tuples()

        return tuples
        
    security.declareProtected( ManageUsers, 'getUserIdForLogin' )
    def getUserIdForLogin(self, login_name):
        return login_name

    security.declareProtected( ManageUsers, 'getLoginForUserId' )
    def getLoginForUserId(self, user_id):
        return user_id

    security.declarePrivate('addUser')
    def addUser(self, user_id, login_name, password):
        existingUser = self.getUserInfo(user_id)

        if existingUser:
            raise KeyError, 'Duplicate user ID: %s' % user_id

        self.sqlCreateUser(username=login_name, password=password)

    security.declarePrivate('removeUser')
    def removeUser( self, user_id ):
        self.sqlRemoveUser(username=user_id)

    security.declarePrivate('updateUserPassword')
    def updateUserPassword(self, user_id, login_name, password):

        existingUser = self.getUserInfo(user_id)
        if existingUser is None:
            raise KeyError, 'Invalid user ID: %s' % user_id

        digested = sha.sha(password).hexdigest()
        # for the time being we will not use encrypted passwords
        #self.sqlUpdateUser(username=login_name, password=digested)
        self.sqlUpdateUser(username=login_name, password=password)

    #
    #   Allow users to change their own password.
    #
    security.declareProtected(SetOwnPassword, 'getOwnUserInfo')
    def getOwnUserInfo(self):
        """
        """
        
        user_id = getSecurityManager().getUser().getId()

        return self.getUserInfo(user_id)
                
    def getSQLQueries(self):
        return _SQL_QUERIES


classImplements(SQLUserManager, 
                IAuthenticationPlugin, 
                IUserEnumerationPlugin,
                IUserAdderPlugin,
                IUserManagement)

InitializeClass(SQLUserManager)


_SQL_QUERIES = (
    (
     "sqlLoadUser", 
     "Retrieve a User", 
     "username", 
     """
SELECT %(username_col)s as username, %(password_col)s as password
FROM %(users_table)s
WHERE %(username_col)s=<dtml-sqlvar username type=string>
     """
    ),
    
    (
     "sqlLoadAllUsers",
     "Retrieve all Users",
     "",
     """
SELECT %(username_col)s as username, %(password_col)s as password
FROM %(users_table)s
     """
    ),
    
    (
     "sqlCreateUser",
     "Create a New User",
     "username password",
     """
INSERT INTO %(users_table)s (%(username_col)s, %(password_col)s)
VALUES (<dtml-sqlvar username type=string>,
        <dtml-sqlvar password type=string>)
     """
    ),
      
    (
     "sqlRemoveUser",
     "Create a New User",
     "username password",
     """
DELETE FROM %(users_table)s
WHERE %(username_col)s=<dtml-sqlvar username type=string>
     """
    ),
    
    (
     "sqlUpdateUser",
     "Update User Info",
     "username password",
     """
UPDATE %(users_table)s
SET %(password_col)s=<dtml-sqlvar password type=string>
WHERE %(username_col)s=<dtml-sqlvar username type=string>
     """
    ),

)
