
__author__  = '''Rocky Burt <rocky@serverzen.com>'''
__docformat__ = 'plaintext'

from Testing import ZopeTestCase
from Products.SQLPASPlugin import utils as sqlpasutils, config as sqlpasconfig
import os
from pysqlite2 import dbapi2 as dbapi

ZopeTestCase.installProduct('PluggableAuthService')
ZopeTestCase.installProduct('PasswordResetTool')
ZopeTestCase.installProduct('SQLPASPlugin')
ZopeTestCase.installProduct('ZSQLMethods')

if sqlpasconfig.TESTING_DB == 'postgresql':
    ZopeTestCase.installProduct('ZPsycopgDA')
else:
    ZopeTestCase.installProduct('ZSQLiteDA')

def _getCreateSQL():
    sql = "CREATE TABLE %s (" % sqlpasconfig.USERS_TABLE
    for name, _type, length, extra in sqlpasconfig.COLUMNS:
        if length is not None:
            sql += "%s %s(%i) %s, " % (name, _type, length, extra)
        else:
            sql += "%s %s %s, " % (name, _type, extra)
    sql = sql[:-2] + ')'
    
    return sql
    

class BaseTestCase(ZopeTestCase.ZopeTestCase):
    
    def _setup(self):
        ZopeTestCase.ZopeTestCase._setup(self)
        self.baseSetup()

    def baseSetup(self):
        # Here we create the connection object according to the
        # configuration
        self.conn = self.setupConnection(self.app, connection_id='db_con')

        # We set up a folder with a PAS in it
        self.app.manage_addFolder('sandbox')
        sandbox = self.app.sandbox
        self.pas = self.setupPAS(sandbox, connection_id='db_con')
        
    def setupConnection(self, container, connection_id):
        if sqlpasconfig.TESTING_DB == 'postgresql':
            # ImportError is nicer than the error that
            # manage_addProduct produces when the Product is not
            # installed.
            import Products.ZPsycopgDA
            zpsycopgda = container.manage_addProduct['ZPsycopgDA']
            addConnection =  zpsycopgda.manage_addZPsycopgConnection
            addConnection(id=connection_id,
                          title='Database Connection',
                          connection_string=sqlpasconfig.TESTING_DB_EXTRA,
                          check=1)
            try:
                self._execute(_getCreateSQL())
            except:
                pass
        elif sqlpasconfig.TESTING_DB == "sqlite":
            # ImportError is nicer than the error that
            # manage_addProduct produces when the Product is not
            # installed.
            import Products.ZSQLiteDA

            dbFile = 'sqlpasplugin-testing.db'
            dbPath = os.path.join(os.getenv('INSTANCE_HOME'), 'var', 'sqlite', dbFile)
    
            if os.path.exists(dbPath):
                os.remove(dbPath)
                
            con = dbapi.connect(dbPath)
            cur = con.cursor()           
    
            self._execute(_getCreateSQL(), cur)
            cur.close()
            con.commit()
            con.close()

            sqliteda = container.manage_addProduct['ZSQLiteDA']
            addSQLite = sqliteda.manage_addZSQLiteConnection
            addSQLite(id=connection_id,
                      title='Database Connection',
                      connection=dbFile,)
        else:
            raise RuntimeError("%r is not a valid value for the TESTING_DB, "
                               "Please correct." % sqlpasconfig.TESTING_DB)

        return getattr(container, connection_id)

    def setupPAS(self, container, connection_id, install_sqlpas=True):
        if hasattr(container.aq_base, 'acl_users'):
            container.manage_delObjects(['acl_users'])
        methods = container.manage_addProduct['PluggableAuthService']
        methods.addPluggableAuthService(REQUEST=None)

        if install_sqlpas:
            sqlpasutils.updatePAS(container, connection_id)
        return container.acl_users

    def _clear(self, call_close_hook=0):
        self.baseClear()
        ZopeTestCase.ZopeTestCase._clear(self, call_close_hook)

    def baseClear(self):
        pass
    
#        try:
#            self._execute("DROP TABLE %s" % sqlpasconfig.USERS_TABLE)
#        except:
#            pass

    def _execute(self, sql, externalCursor=None):
        
        db = None
        if externalCursor:
            cursor = externalCursor
        else:
            conn = self.conn
            db = conn().db
            cursor = db.cursor()
        
        try:
            sql = ' '.join(sql.split('\n')).strip()
            sqlpasutils.log('executing sql [%s]' % sql)
            cursor.execute(sql)
        finally:
            if db:
                db.commit()
            if not externalCursor:
                try:
                    cursor.close()
                except:
                    pass

    def getPAS(self):
        return self.pas

        
