import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.SQLPASPlugin.tests import basetestcase
from Products.SQLPASPlugin.plugins import propertyprovider
import AccessControl


class TestPropertyProvider(basetestcase.BaseTestCase):

    def afterSetUp(self):
        self.username = 'joe'
        self.password = 'password'
        self.source_users = self.getPAS().source_users
        self.source_properties = self.getPAS().source_properties
        
    def testGetPropertiesForUser(self):
        sm = AccessControl.getSecurityManager()
        user = sm.getUser()

        self.source_users.doAddUser(user.getUserName(), self.password)
        props = self.source_properties.getPropertiesForUser(user)
        
        self.assertNotEqual(len(props.propertyIds()), 0)
        
        user_name_column = self.source_properties.getProperty('users_col_username')
        u = props.getProperty(user_name_column)
        self.assertEqual(u, user.getUserName())
        
        user_pass_column = self.source_properties.getProperty('users_col_password')
        p = props.getProperty(user_pass_column)
        self.assertEqual(p, self.password)

        self.source_users.removeUser(user.getUserName())
        
    def testUpdateUserInfo(self):
        sm = AccessControl.getSecurityManager()
        user = sm.getUser()
        
        info = {
            'firstname': 'Joe',
            'lastname': 'Schmoe',
            'email': 'joe@localhost.localdomain'
        }
        self.source_users.doAddUser(user.getUserName(), self.password)
        self.source_properties.updateUserInfo(user=user, set_id=None, set_info=info)
        
        props = self.source_properties.getPropertiesForUser(user)

        self.assertEqual(props.getProperty('firstname'), info['firstname'])
        self.assertEqual(props.getProperty('lastname'), info['lastname'])
        self.assertEqual(props.getProperty('email'), info['email'])

        self.source_users.removeUser(user.getUserName())

    def testStructure(self):
        self.assertEqual(len(self.source_properties.objectIds()), 
                         len(propertyprovider._SQL_QUERIES))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPropertyProvider))
    return suite

if __name__ == '__main__':
    framework()
