import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.SQLPASPlugin.tests import basetestcase
from Products.SQLPASPlugin.plugins import usermanager


class TestUserManager(basetestcase.BaseTestCase):

    def afterSetUp(self):
        self.username = 'joe'
        self.password = 'password'
        self.source_users = self.getPAS().source_users

    def testDoAddUser(self):
        self.source_users.doAddUser(self.username, self.password)
        ret = self.source_users.enumerateUsers(id=self.username, exact_match=True)
        self.assertEqual(len(ret), 1)
        self.source_users.removeUser(self.username)
    
    def testRemoveUser(self):
        ret = self.source_users.enumerateUsers(id=self.username, exact_match=True)
        self.assertEqual(len(ret), 0)
        self.source_users.doAddUser(self.username, self.password)
        self.source_users.removeUser(self.username)
        ret = self.source_users.enumerateUsers(id=self.username, exact_match=True)
        self.assertEqual(len(ret), 0)
    
    def testEnumerateUsers(self):
        ret = self.source_users.enumerateUsers()
        self.assertEqual(len(ret), 0)
        count = 10
        for x in range(count):
            username = 'user_%i' % x
            self.source_users.doAddUser(username, self.password)
        ret = self.source_users.enumerateUsers()
        self.assertEqual(len(ret), count)

        ret = self.source_users.enumerateUsers(id='user_1', exact_match=True)
        self.assertEqual(len(ret), 1)

        ret = self.source_users.enumerateUsers(login='user_1', exact_match=True)
        self.assertEqual(len(ret), 1)
        
        ret = self.source_users.enumerateUsers(max_results=5)
        self.assertEqual(len(ret), 5)
        
        ret = self.source_users.enumerateUsers(max_results=20)
        self.assertEqual(len(ret), count)

        for x in range(count):
            username = 'user_%i' % x
            self.source_users.removeUser(username)
    
    def testAuthenticateCredentials(self):
        auth = self.source_users.authenticateCredentials({'login': self.username, 
                                                          'password': self.password})
        self.assertEqual(auth, None)
        
        self.source_users.doAddUser(self.username, self.password)
        auth = self.source_users.authenticateCredentials({'login': self.username, 
                                                          'password': self.password})
        self.assertEqual(auth, (self.username, self.username))
        self.source_users.removeUser(self.username)
        
    def testStructure(self):
        self.assertEqual(len(self.source_users.objectIds()), 
                         len(usermanager._SQL_QUERIES))

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestUserManager))
    return suite

if __name__ == '__main__':
    framework()
